from urllib import parse

from django import forms
from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse
from django.utils.html import format_html
from edc_action_item.models import ActionType
from edc_adverse_event.form_validator_mixins import (
    RequiresDeathReportFormValidatorMixin,
)
from edc_consent.constants import CONSENT_WITHDRAWAL
from edc_constants.constants import DEAD, DELIVERY, PREGNANCY, TOXICITY
from edc_form_validators import INVALID_ERROR, FormValidator
from edc_ltfu.constants import LTFU
from edc_ltfu.modelform_mixins import RequiresLtfuFormValidatorMixin
from edc_offstudy.constants import COMPLETED_FOLLOWUP
from edc_offstudy.utils import OffstudyError
from edc_prn.modelform_mixins import PrnFormValidatorMixin
from edc_transfer.constants import TRANSFERRED
from edc_utils import formatted_date
from edc_visit_schedule.constants import MONTH36
from edc_visit_schedule.utils import off_all_schedules_or_raise

from ..constants import (
    CLINICAL_WITHDRAWAL,
    INVESTIGATOR_DECISION,
    OFFSTUDY_MEDICATION_ACTION,
)


class EndOfStudyFormValidator(
    RequiresDeathReportFormValidatorMixin,
    RequiresLtfuFormValidatorMixin,
    PrnFormValidatorMixin,
    FormValidator,
):
    death_report_model = "meta_ae.deathreport"
    ltfu_model = None

    def clean(self):

        self.confirm_off_all_schedules()
        self.validate_study_medication_status()
        self.validate_offstudy_datetime_against_last_seen_date()

        self.validate_completed_36m()

        self.required_if(DEAD, field="offstudy_reason", field_required="death_date")
        self.validate_death_report_if_deceased()

        self.required_if(PREGNANCY, field="offstudy_reason", field_required="pregnancy_date")
        self.validate_pregnancy()

        self.required_if(DELIVERY, field="offstudy_reason", field_required="delivery_date")
        self.validate_delivery()

        self.required_if(TRANSFERRED, field="offstudy_reason", field_required="transfer_date")
        self.validate_transfer()

        self.required_if(LTFU, field="offstudy_reason", field_required="ltfu_date")
        self.validate_ltfu()

        self.applicable_if(
            TOXICITY, field="offstudy_reason", field_applicable="toxicity_withdrawal_reason"
        )
        self.validate_other_specify(
            field="toxicity_withdrawal_reason",
            other_specify_field="toxicity_withdrawal_reason_other",
        )

        self.applicable_if(
            CLINICAL_WITHDRAWAL,
            field="offstudy_reason",
            field_applicable="clinical_withdrawal_reason",
        )

        self.validate_other_specify(
            other_stored_value=INVESTIGATOR_DECISION,
            field="clinical_withdrawal_reason",
            other_specify_field="clinical_withdrawal_investigator_decision",
        )

        self.validate_other_specify(
            field="clinical_withdrawal_reason",
            other_specify_field="clinical_withdrawal_reason_other",
        )

        self.required_if(
            CONSENT_WITHDRAWAL,
            field="offstudy_reason",
            field_required="consent_withdrawal_reason",
        )

    def validate_completed_36m(self):
        if (
            self.cleaned_data.get("offstudy_reason")
            and self.cleaned_data.get("offstudy_reason").name == COMPLETED_FOLLOWUP
        ):
            subject_visit_model_cls = django_apps.get_model("meta_subject.subjectvisit")
            try:
                subject_visit_model_cls.objects.get(
                    subject_identifier=self.subject_identifier,
                    visit_code=MONTH36,
                    visit_code_sequence=0,
                )
            except ObjectDoesNotExist:
                self.raise_validation_error(
                    {"offstudy_reason": "Invalid. 36 month visit has not been submitted."},
                    INVALID_ERROR,
                )

    def validate_pregnancy(self):
        if (
            self.cleaned_data.get("offstudy_reason")
            and self.cleaned_data.get("offstudy_reason").name == PREGNANCY
        ):
            delivery_model_cls = django_apps.get_model("meta_subject.delivery")
            pregnancy_notification_model_cls = django_apps.get_model(
                "meta_prn.pregnancynotification"
            )
            try:
                delivery_model_cls.objects.get(
                    subject_visit__subject_identifier=self.subject_identifier,
                )
            except ObjectDoesNotExist:
                pass
            else:
                self.raise_validation_error(
                    {
                        "offstudy_reason": (
                            f"Invalid. {delivery_model_cls._meta.verbose_name} "
                            "has been submitted."
                        )
                    },
                    INVALID_ERROR,
                )
            try:
                obj = pregnancy_notification_model_cls.objects.get(
                    subject_identifier=self.subject_identifier,
                )
            except ObjectDoesNotExist:
                self.raise_validation_error(
                    {
                        "offstudy_reason": (
                            f"Invalid. {pregnancy_notification_model_cls._meta.verbose_name} "
                            "has not been submitted."
                        )
                    },
                    INVALID_ERROR,
                )
            else:
                if (obj.bhcg_date or obj.report_datetime.date()) != self.cleaned_data.get(
                    "pregnancy_date"
                ):
                    dt = formatted_date((obj.bhcg_date or obj.report_datetime.date()))
                    self.raise_validation_error(
                        {
                            "pregnancy_date": (
                                "Invalid. Does not match date on "
                                f"{pregnancy_notification_model_cls._meta.verbose_name}. "
                                f"Expected {dt}."
                            )
                        },
                        INVALID_ERROR,
                    )

    def validate_delivery(self):
        if (
            self.cleaned_data.get("offstudy_reason")
            and self.cleaned_data.get("offstudy_reason").name == DELIVERY
        ):
            delivery_model_cls = django_apps.get_model("meta_subject.delivery")
            try:
                obj = delivery_model_cls.objects.get(
                    subject_visit__subject_identifier=self.subject_identifier,
                )
            except ObjectDoesNotExist:
                self.raise_validation_error(
                    {
                        "offstudy_reason": (
                            f"Invalid. {delivery_model_cls._meta.verbose_name} "
                            "has not been submitted."
                        )
                    },
                    INVALID_ERROR,
                )
            else:
                if (
                    obj.delivery_datetime.date() or obj.report_datetime.date()
                ) != self.cleaned_data.get("delivery_date"):
                    dt = formatted_date(
                        (obj.delivery_datetime.date() or obj.report_datetime.date())
                    )
                    self.raise_validation_error(
                        {
                            "delivery_date": (
                                "Invalid. Does not match date on "
                                f"{delivery_model_cls._meta.verbose_name}. "
                                f"Expected {dt}."
                            )
                        },
                        INVALID_ERROR,
                    )

    def validate_study_medication_status(self):
        off_study_medication_model_cls = django_apps.get_model("meta_prn.offstudymedication")
        try:
            off_study_medication_model_cls.objects.get(
                subject_identifier=self.subject_identifier
            )
        except ObjectDoesNotExist:
            action_type = ActionType.objects.get(name=OFFSTUDY_MEDICATION_ACTION)
            url = reverse("edc_action_item_admin:edc_action_item_actionitem_add")
            data = dict(
                subject_identifier=self.subject_identifier,
                action_type=str(action_type.id),
            )
            query_string = "&".join(
                [
                    "next=meta_dashboard:subject_dashboard_url,subject_identifier",
                    parse.urlencode(data),
                ]
            )

            self.raise_validation_error(
                format_html(
                    "Participant is reported to be on study medication. Complete "
                    f'<a href="{url}?{query_string}" title="Add new form">'
                    f"{off_study_medication_model_cls._meta.verbose_name}</A> "
                    "PRN action and try again."
                ),
                INVALID_ERROR,
            )

    def confirm_off_all_schedules(self):
        try:
            off_all_schedules_or_raise(
                subject_identifier=self.cleaned_data.get("subject_identifier"),
            )
        except OffstudyError as e:
            self.raise_validation_error(str(e), INVALID_ERROR)

    def validate_offstudy_datetime_against_last_seen_date(self):
        if self.cleaned_data.get("offstudy_datetime") and self.cleaned_data.get(
            "last_seen_date"
        ):
            if (
                self.cleaned_data.get("last_seen_date")
                > self.cleaned_data.get("offstudy_datetime").date()
            ):
                raise forms.ValidationError(
                    {"last_seen_date": "Invalid. May not be after termination date"}
                )

    def validate_transfer(self):
        if (
            self.cleaned_data.get("offstudy_reason")
            and self.cleaned_data.get("offstudy_reason").name == TRANSFERRED
        ):
            transfer_model_cls = django_apps.get_model("meta_prn.subjecttransfer")
            try:
                obj = transfer_model_cls.objects.get(
                    subject_visit__subject_identifier=self.subject_identifier,
                )
            except ObjectDoesNotExist:
                self.raise_validation_error(
                    {
                        "offstudy_reason": (
                            f"Invalid. {transfer_model_cls._meta.verbose_name} "
                            "has not been submitted."
                        )
                    },
                    INVALID_ERROR,
                )
            else:
                if obj.transfer_date.date() != self.cleaned_data.get("transfer_date"):
                    dt = formatted_date(obj.transfer_date)
                    self.raise_validation_error(
                        {
                            "transfer_date": (
                                "Invalid. Does not match date on "
                                f"{transfer_model_cls._meta.verbose_name}. Expected {dt}."
                            )
                        },
                        INVALID_ERROR,
                    )



from edc_model.models import BaseUuidModel
from edc_qol.model_mixins import Sf12ModelMixin

from ..model_mixins import CrfModelMixin


class Sf12(Sf12ModelMixin, CrfModelMixin, BaseUuidModel):
    class Meta(Sf12ModelMixin.Meta, CrfModelMixin.Meta, BaseUuidModel.Meta):
        pass


# TODO: add proxy model and two options Swahili and English



from edc_model.models import BaseUuidModel
from edc_sites.models import SiteModelMixin
from edc_visit_schedule.model_mixins import OnScheduleModelMixin


class OnSchedule(SiteModelMixin, OnScheduleModelMixin, BaseUuidModel):

    """A model used by the system. Auto-completed by subject_consent."""

    class Meta(OnScheduleModelMixin.Meta, BaseUuidModel.Meta):
        pass


class OnSchedulePregnancy(SiteModelMixin, OnScheduleModelMixin, BaseUuidModel):
    class Meta(OnScheduleModelMixin.Meta, BaseUuidModel.Meta):
        pass


class OnSchedulePostnatal(SiteModelMixin, OnScheduleModelMixin, BaseUuidModel):
    class Meta(OnScheduleModelMixin.Meta):
        pass



from edc_sites.single_site import SingleSite

fqdn = "meta3.clinicedc.org"


all_sites = {
    "tanzania": (
        SingleSite(
            10,
            "hindu_mandal",
            title="Hindu Mandal Hospital",
            country="tanzania",
            country_code="tz",
            domain=f"hindu-mandal.tz.{fqdn}",
        ),
        SingleSite(
            20,
            "amana",
            title="Amana Hospital",
            country="tanzania",
            country_code="tz",
            domain=f"amana.tz.{fqdn}",
        ),
        SingleSite(
            30,
            "temeke",
            title="Temeke Hospital",
            country="tanzania",
            country_code="tz",
            domain=f"temeke.tz.{fqdn}",
        ),
        SingleSite(
            40,
            "mwananyamala",
            title="Mwananyamala Hospital",
            country="tanzania",
            country_code="tz",
            domain=f"mwananyamala.tz.{fqdn}",
        ),
        SingleSite(
            50,
            "mbagala",
            title="Mbagala Hospital",
            country="tanzania",
            country_code="tz",
            domain=f"mbagala.tz.{fqdn}",
        ),
        SingleSite(
            60,
            "mnazi_moja",
            title="Mnazi Moja Hospital",
            country="tanzania",
            country_code="tz",
            domain=f"mnazi-moja.tz.{fqdn}",
        ),
    ),
}



# Generated by Django 3.2.13 on 2022-09-06 22:57

import uuid

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0143_auto_20220826_0406"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalstudymedication",
            old_name="order_next",
            new_name="order_or_update_next",
        ),
        migrations.RenameField(
            model_name="studymedication",
            old_name="order_next",
            new_name="order_or_update_next",
        ),
        migrations.RenameField(
            model_name="historicalstudymedication",
            old_name="refill_date",
            new_name="refill_start_datetime",
        ),
        migrations.RenameField(
            model_name="studymedication",
            old_name="refill_date",
            new_name="refill_start_datetime",
        ),
    ]



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from edc_consent import ConsentModelWrapperMixin
from edc_constants.constants import PENDING
from edc_model_wrapper import ModelWrapper
from edc_subject_model_wrappers import SubjectConsentModelWrapper as BaseModelWrapper
from edc_utils import get_utcnow


class SubjectConsentModelWrapper(BaseModelWrapper):
    @property
    def querystring(self):
        return (
            f"cancel=meta_dashboard:screening_listboard_url,"
            f"screening_identifier&{super().querystring}"
        )


class SubjectScreeningModelWrapper(ConsentModelWrapperMixin, ModelWrapper):

    consent_model_wrapper_cls = SubjectConsentModelWrapper
    model = "meta_screening.subjectscreening"
    next_url_attrs = ["screening_identifier"]
    next_url_name = "screening_listboard_url"
    querystring_attrs = ["gender"]

    @property
    def create_consent_options(self):
        options = super().create_consent_options
        options.update(screening_identifier=self.object.screening_identifier)
        return options

    @property
    def consent_options(self):
        return dict(screening_identifier=self.object.screening_identifier)

    @property
    def consent_model_obj(self):
        consent_model_cls = django_apps.get_model(self.consent_model_wrapper_cls.model)
        try:
            return consent_model_cls.objects.get(**self.consent_options)
        except ObjectDoesNotExist:
            return None

    @property
    def human_screening_identifier(self):
        human = None
        if self.screening_identifier:
            human = f"{self.screening_identifier[0:4]}-{self.screening_identifier[4:]}"
        return human or self.screening_identifier

    @property
    def href_p1(self):
        return self.href.replace("subjectscreening", "screeningpartone")

    @property
    def href_p2(self):
        return self.href.replace("subjectscreening", "screeningparttwo")

    @property
    def href_p3(self):
        return self.href.replace("subjectscreening", "screeningpartthree")

    @property
    def repeat_due_in_days(self):
        if self.repeat_glucose_performed == PENDING and self.object.ogtt_datetime:
            return (get_utcnow() - self.object.ogtt_datetime).days
        return 0


class ScreeningPartOneModelWrapper(SubjectScreeningModelWrapper):

    model = "meta_screening.screeningpartone"

    @property
    def href_p1(self):
        return self.href

    @property
    def href_p2(self):
        return self.href.replace("screeningpartone", "screeningparttwo")

    @property
    def href_p3(self):
        return self.href.replace("screeningpartone", "screeningpartthree")



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset

from ..admin_site import meta_subject_admin
from ..forms import GlucoseForm
from ..models import Glucose
from .modeladmin import CrfModelAdmin


@admin.register(Glucose, site=meta_subject_admin)
class GlucoseAdmin(CrfModelAdmin):

    form = GlucoseForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Fasting",
            {
                "fields": (
                    "fasting",
                    "fasting_duration_str",
                )
            },
        ),
        (
            "FBG",
            {
                "fields": (
                    "fbg_performed",
                    "fbg_not_performed_reason",
                    "fbg_datetime",
                    "fbg_value",
                    "fbg_units",
                )
            },
        ),
        (
            "OGTT",
            {
                "fields": (
                    "ogtt_performed",
                    "ogtt_not_performed_reason",
                    "ogtt_base_datetime",
                    "ogtt_datetime",
                    "ogtt_value",
                    "ogtt_units",
                )
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "fasting": admin.VERTICAL,
        "fbg_units": admin.VERTICAL,
        "fbg_performed": admin.VERTICAL,
        "ogtt_performed": admin.VERTICAL,
        "ogtt_units": admin.VERTICAL,
    }



# Generated by Django 2.2.11 on 2020-03-25 06:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_consent", "0002_auto_20191024_1000"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="identity_type",
            field=models.CharField(
                choices=[
                    ("country_id", "Country ID number"),
                    ("drivers", "Driver's license"),
                    ("passport", "Passport"),
                    ("hospital_no", "Hospital number"),
                    ("country_id_rcpt", "Country ID receipt"),
                    ("mobile_no", "Mobile number"),
                    ("OTHER", "Other"),
                ],
                max_length=25,
                verbose_name="What type of identity number is this?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="identity_type",
            field=models.CharField(
                choices=[
                    ("country_id", "Country ID number"),
                    ("drivers", "Driver's license"),
                    ("passport", "Passport"),
                    ("hospital_no", "Hospital number"),
                    ("country_id_rcpt", "Country ID receipt"),
                    ("mobile_no", "Mobile number"),
                    ("OTHER", "Other"),
                ],
                max_length=25,
                verbose_name="What type of identity number is this?",
            ),
        ),
    ]



# Generated by Django 3.0.6 on 2020-05-24 16:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0019_auto_20200120_2256"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_creatinine",
            field=models.DecimalField(
                decimal_places=4,
                help_text="μmol/L (micromoles/L) (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (creatinine)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_creatinine",
            field=models.DecimalField(
                decimal_places=4,
                help_text="μmol/L (micromoles/L) (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (creatinine)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_creatinine",
            field=models.DecimalField(
                decimal_places=4,
                help_text="μmol/L (micromoles/L) (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (creatinine)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_creatinine",
            field=models.DecimalField(
                decimal_places=4,
                help_text="μmol/L (micromoles/L) (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (creatinine)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_creatinine",
            field=models.DecimalField(
                decimal_places=4,
                help_text="μmol/L (micromoles/L) (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (creatinine)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
    ]



from .end_of_study import EndOfStudy
from .loss_to_followup import LossToFollowup
from .off_study_medication import OffStudyMedication
from .offschedule import OffSchedule, OffSchedulePostnatal, OffSchedulePregnancy
from .onschedule import OnSchedule, OnSchedulePostnatal, OnSchedulePregnancy
from .pregnancy_notification import PregnancyNotification
from .protocol_incident import ProtocolIncident
from .signals import (
    update_schedule_on_pregnancy_notification_post_save,
    update_urine_pregnancy_on_pregnancy_notification_on_post_save,
)
from .subject_transfer import SubjectTransfer



from django.core.exceptions import ValidationError
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from edc_action_item import ActionItemDeleteError, delete_action_item
from edc_constants.constants import YES
from edc_pharmacy.exceptions import PrescriptionAlreadyExists
from edc_pharmacy.prescribe import create_prescription
from edc_randomization.site_randomizers import site_randomizers
from edc_visit_schedule.site_visit_schedules import site_visit_schedules

from meta_edc.meta_version import get_meta_version
from meta_screening.models import SubjectScreening
from meta_subject.models import SubjectVisit

from ..action_items import ReconsentAction
from .subject_consent import SubjectConsent


@receiver(
    post_save,
    weak=False,
    sender=SubjectConsent,
    dispatch_uid="subject_consent_on_post_save",
)
def subject_consent_on_post_save(sender, instance, raw, created, **kwargs):
    """Creates an onschedule instance for this consented subject, if
    it does not exist.
    """
    if not raw:
        if not created:
            _, schedule = site_visit_schedules.get_by_onschedule_model("meta_prn.onschedule")
            schedule.refresh_schedule(subject_identifier=instance.subject_identifier)
        else:
            subject_screening = SubjectScreening.objects.get(
                screening_identifier=instance.screening_identifier
            )
            subject_screening.subject_identifier = instance.subject_identifier
            subject_screening.consented = True
            subject_screening.save_base(update_fields=["subject_identifier", "consented"])

            # randomize
            site_randomizers.randomize(
                get_meta_version(),
                subject_identifier=instance.subject_identifier,
                report_datetime=instance.consent_datetime,
                site=instance.site,
                user=instance.user_created,
                gender=instance.gender,
            )

            # put subject on schedule
            _, schedule = site_visit_schedules.get_by_onschedule_model("meta_prn.onschedule")
            schedule.put_on_schedule(
                subject_identifier=instance.subject_identifier,
                onschedule_datetime=instance.consent_datetime,
            )

            # All refills are created against this prescription
            try:
                create_prescription(
                    subject_identifier=instance.subject_identifier,
                    report_datetime=instance.consent_datetime,
                    medications=[instance.study_medication_name],
                    randomizer_name=get_meta_version(),
                    site=instance.site,
                )
            except PrescriptionAlreadyExists:
                pass

        # create / delete action for reconsent
        if instance.completed_by_next_of_kin == YES:
            ReconsentAction(subject_identifier=instance.subject_identifier)
        else:
            try:
                delete_action_item(
                    action_cls=ReconsentAction,
                    subject_identifier=instance.subject_identifier,
                )
            except ActionItemDeleteError:
                pass


@receiver(
    post_delete,
    weak=False,
    sender=SubjectConsent,
    dispatch_uid="subject_consent_on_post_delete",
)
def subject_consent_on_post_delete(sender, instance, using, **kwargs):
    """Updates/Resets subject screening."""
    # don't allow if subject visits exist. This should be caught
    # in the ModelAdmin delete view
    if SubjectVisit.objects.filter(subject_identifier=instance.subject_identifier).exists():
        raise ValidationError("Unable to delete consent. Visit data exists.")

    _, schedule = site_visit_schedules.get_by_onschedule_model("meta_prn.onschedule")
    schedule.take_off_schedule(
        subject_identifier=instance.subject_identifier,
        offschedule_datetime=instance.consent_datetime,
    )

    # update subject screening
    subject_screening = SubjectScreening.objects.get(
        screening_identifier=instance.screening_identifier
    )
    subject_screening.consented = False
    subject_screening.subject_identifier = subject_screening.subject_screening_as_pk
    subject_screening.save()



from django.contrib import admin
from django.utils.html import format_html
from edc_adverse_event.modeladmin_mixins import AeInitialModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_notification.utils import get_email_contacts

from ..admin_site import meta_ae_admin
from ..forms import AeInitialForm
from ..models import AeInitial


@admin.register(AeInitial, site=meta_ae_admin)
class AeInitialAdmin(AeInitialModelAdminMixin, SimpleHistoryAdmin):

    form = AeInitialForm
    email_contact = get_email_contacts("ae_reports")
    additional_instructions = format_html(
        "Complete the initial AE report and forward to the TMG. "
        'Email to <a href="mailto:{}">{}</a>',
        email_contact,
        email_contact,
    )



from django import forms
from edc_consent.form_validators import ConsentFormValidatorMixin
from edc_constants.constants import MALE, YES
from edc_form_validators import FormValidator
from edc_prn.modelform_mixins import PrnFormValidatorMixin


class ScreeningPartOneFormValidator(
    ConsentFormValidatorMixin, PrnFormValidatorMixin, FormValidator
):
    def clean(self):

        self.get_consent_for_period_or_raise()

        if (
            not self.cleaned_data.get("screening_consent")
            or self.cleaned_data.get("screening_consent") != YES
        ):
            raise forms.ValidationError(
                {
                    "screening_consent": (
                        "You may NOT screen this subject without their verbal consent."
                    )
                }
            )

        self.applicable_if(YES, field="hiv_pos", field_applicable="art_six_months")

        self.applicable_if(YES, field="hiv_pos", field_applicable="on_rx_stable")

        self.applicable_if(YES, field="hiv_pos", field_applicable="vl_undetectable")

        # TODO: what if FEMALE and not applicable??

        self.not_applicable_if(
            MALE, field="gender", field_applicable="pregnant", inverse=False
        )

        self.required_if(
            YES, field="unsuitable_for_study", field_required="reasons_unsuitable"
        )



# Generated by Django 3.2.7 on 2021-11-04 11:47

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_protocol_incident", "0003_auto_20211104_1456"),
        ("meta_prn", "0014_auto_20211003_1709"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalprotocoldeviationviolation",
            name="violation",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_protocol_incident.protocolviolations",
                verbose_name="Type of violation",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocoldeviationviolation",
            name="violation_type",
            field=models.CharField(
                blank=True,
                choices=[
                    (
                        "failure_to_obtain_informed_consent",
                        "Failure to obtain informed consent",
                    ),
                    (
                        "enrollment_of_ineligible_patient",
                        "Enrollment of ineligible patient",
                    ),
                    (
                        "screening_procedure not done",
                        "Screening procedure required by protocol not done",
                    ),
                    (
                        "screening_or_on-study_procedure",
                        "Screening or on-study procedure/lab work required not done",
                    ),
                    (
                        "incorrect_research_treatment",
                        "Incorrect research treatment given to patient",
                    ),
                    (
                        "procedure_not_completed",
                        "On-study procedure required by protocol not completed",
                    ),
                    ("visit_non-compliance", "Visit non-compliance"),
                    ("medication_stopped_early", "Medication stopped early"),
                    ("medication_noncompliance", "Medication_noncompliance"),
                    (
                        "national_regulations_not_met",
                        "Standard WPD, ICH-GCP, local/national regulations not met",
                    ),
                    ("OTHER", "Other"),
                    ("N/A", "Not applicable"),
                ],
                default="QUESTION_RETIRED",
                max_length=75,
                null=True,
                verbose_name="Type of violation",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocoldeviationviolation",
            name="violation_type_other",
            field=models.CharField(
                blank=True,
                default="QUESTION_RETIRED",
                max_length=75,
                null=True,
                verbose_name="If other, please specify",
            ),
        ),
        migrations.AlterField(
            model_name="protocoldeviationviolation",
            name="violation",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_protocol_incident.protocolviolations",
                verbose_name="Type of violation",
            ),
        ),
        migrations.AlterField(
            model_name="protocoldeviationviolation",
            name="violation_type",
            field=models.CharField(
                blank=True,
                choices=[
                    (
                        "failure_to_obtain_informed_consent",
                        "Failure to obtain informed consent",
                    ),
                    (
                        "enrollment_of_ineligible_patient",
                        "Enrollment of ineligible patient",
                    ),
                    (
                        "screening_procedure not done",
                        "Screening procedure required by protocol not done",
                    ),
                    (
                        "screening_or_on-study_procedure",
                        "Screening or on-study procedure/lab work required not done",
                    ),
                    (
                        "incorrect_research_treatment",
                        "Incorrect research treatment given to patient",
                    ),
                    (
                        "procedure_not_completed",
                        "On-study procedure required by protocol not completed",
                    ),
                    ("visit_non-compliance", "Visit non-compliance"),
                    ("medication_stopped_early", "Medication stopped early"),
                    ("medication_noncompliance", "Medication_noncompliance"),
                    (
                        "national_regulations_not_met",
                        "Standard WPD, ICH-GCP, local/national regulations not met",
                    ),
                    ("OTHER", "Other"),
                    ("N/A", "Not applicable"),
                ],
                default="QUESTION_RETIRED",
                max_length=75,
                null=True,
                verbose_name="Type of violation",
            ),
        ),
        migrations.AlterField(
            model_name="protocoldeviationviolation",
            name="violation_type_other",
            field=models.CharField(
                blank=True,
                default="QUESTION_RETIRED",
                max_length=75,
                null=True,
                verbose_name="If other, please specify",
            ),
        ),
    ]



from django.db import models
from edc_model.models import BaseUuidModel
from edc_pharmacy.model_mixins import StudyMedicationCrfModelMixin

from ..model_mixins import CrfModelMixin


class StudyMedication(StudyMedicationCrfModelMixin, CrfModelMixin, BaseUuidModel):

    roundup_divisible_by = models.IntegerField(default=32)

    class Meta(
        StudyMedicationCrfModelMixin.Meta,
        CrfModelMixin.Meta,
        BaseUuidModel.Meta,
    ):
        pass



# Generated by Django 3.2.11 on 2022-07-19 23:16

from django.db import migrations
from django.db.models import F


def update_egfr_drop_field_to_percent(apps, schema_editor):
    rft_model_cls = apps.get_model("meta_subject.historicalbloodresultsrft")
    rft_model_cls.objects.all().update(egfr_drop_value=F("egfr_drop_value") * 100)
    rft_model_cls = apps.get_model("meta_subject.bloodresultsrft")
    rft_model_cls.objects.all().update(egfr_drop_value=F("egfr_drop_value") * 100)


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0129_auto_20220720_0108"),
    ]

    operations = [migrations.RunPython(update_egfr_drop_field_to_percent)]



from .ae_initial_model_wrapper import AeInitialModelWrapper
from .death_report_model_wrapper import DeathReportModelWrapper
from .subject_refusal_model_wrapper import SubjectRefusalModelWrapper
from .subject_screening_model_wrapper import (
    ScreeningPartOneModelWrapper,
    SubjectConsentModelWrapper,
    SubjectScreeningModelWrapper,
)



# Generated by Django 4.1.2 on 2022-10-05 18:37

from django.db import migrations, models
import edc_model.validators.date


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0155_auto_20220929_1742"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsins",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslipid",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result assay date and time",
            ),
        ),
    ]



from django import forms
from edc_adverse_event.form_validator_mixins import DeathReportFormValidatorMixin
from edc_adverse_event.modelform_mixins import DeathReportModelFormMixin
from edc_constants.constants import OTHER
from edc_form_validators import FormValidator

from ..constants import HOSPITAL_CLINIC
from ..models import DeathReport


class DeathReportFormValidator(DeathReportFormValidatorMixin, FormValidator):
    def clean(self):
        self.required_if(
            HOSPITAL_CLINIC,
            field="death_location",
            field_required="hospital_name",
        )

        self.validate_other_specify(
            field="informant_relationship",
            other_specify_field="other_informant_relationship",
        )

        other = self.cause_of_death_model_cls.objects.get(name=OTHER)

        self.validate_other_specify(
            field="cause_of_death",
            other_specify_field="cause_of_death_other",
            other_stored_value=other.name,
        )

        self.validate_other_specify(
            field="secondary_cause_of_death",
            other_specify_field="secondary_cause_of_death_other",
            other_stored_value=other.name,
        )


class DeathReportForm(DeathReportModelFormMixin, forms.ModelForm):

    form_validator_cls = DeathReportFormValidator

    class Meta(DeathReportModelFormMixin.Meta):
        model = DeathReport
        fields = "__all__"



from django.apps import apps as django_apps
from django.test import TestCase
from edc_sites.models import CurrentSiteManager as DefaultCurrentSiteManager
from edc_visit_tracking.managers import CrfCurrentSiteManager, VisitCurrentSiteManager


class TestManagers(TestCase):
    def test_models(self):
        app_label = "meta_subject"
        app_config = django_apps.get_app_config(app_label)
        inlines = [f"{app_label}.birthoutcomes", f"{app_label}.otherarvregimensdetail"]
        visit_model = f"{app_label}.subjectvisit"
        for model_cls in app_config.get_models():
            if model_cls._meta.label_lower in inlines:
                self.assertEqual(
                    model_cls._default_manager.__class__,
                    DefaultCurrentSiteManager,
                    msg=f"Model is {model_cls}",
                )
            elif model_cls._meta.label_lower == visit_model:
                self.assertEqual(
                    model_cls._default_manager.__class__,
                    VisitCurrentSiteManager,
                    msg=f"Model is {model_cls}",
                )
            elif "historical" not in model_cls._meta.label_lower:
                self.assertEqual(
                    model_cls._default_manager.__class__,
                    CrfCurrentSiteManager,
                    msg=f"Model is {model_cls}",
                )



from django.contrib import admin
from django.template.loader import render_to_string
from django.urls.base import reverse
from django.urls.exceptions import NoReverseMatch
from django.utils.html import format_html
from django_audit_fields.admin import audit_fieldset_tuple
from edc_constants.constants import YES
from edc_dashboard.url_names import url_names
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_screening.utils import format_reasons_ineligible

from ..admin_site import meta_screening_admin
from ..eligibility import MetaEligibility
from ..forms import SubjectScreeningForm
from ..models import SubjectScreening
from .fieldsets import (
    calculated_values_fieldset,
    get_p3_screening_appt_update_fields,
    get_part_one_fieldset,
    get_part_three_fieldset,
    get_part_two_fieldset,
)
from .list_filters import EligibilityPending, P3LtfuListFilter


@admin.register(SubjectScreening, site=meta_screening_admin)
class SubjectScreeningAdmin(ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):

    form = SubjectScreeningForm
    list_per_page = 15
    post_url_on_delete_name = "screening_listboard_url"
    subject_listboard_url_name = "screening_listboard_url"

    additional_instructions = (
        "Patients must meet ALL of the inclusion criteria and NONE of the "
        "exclusion criteria in order to proceed to the final screening stage"
    )

    fieldsets = (
        get_part_one_fieldset(),
        get_part_two_fieldset(),
        get_p3_screening_appt_update_fields(),
        get_part_three_fieldset(),
        calculated_values_fieldset,
        audit_fieldset_tuple,
    )

    list_display = (
        "screening_identifier",
        "eligibility_status",
        "demographics",
        "reasons",
        "report_datetime",
        "p3_appt",
        "p3_repeat_appt",
        "user_created",
        "created",
    )

    list_filter = (
        "report_datetime",
        "part_three_report_datetime",
        EligibilityPending,
        P3LtfuListFilter,
        "p3_ltfu",
        "gender",
        "eligible",
        "eligible_part_one",
        "eligible_part_two",
        "eligible_part_three",
        "consented",
        "refused",
    )

    search_fields = (
        "screening_identifier",
        "subject_identifier",
        "hospital_identifier",
        "initials",
        "reasons_ineligible",
    )

    readonly_fields = (
        "calculated_bmi_value",
        "calculated_egfr_value",
        "converted_fbg_value",
        "converted_fbg2_value",
        "converted_creatinine_value",
        "converted_ogtt_value",
        "converted_ogtt2_value",
        "inclusion_a",
        "inclusion_b",
        "inclusion_c",
        "inclusion_d",
    )

    radio_fields = {
        "acute_condition": admin.VERTICAL,
        "acute_metabolic_acidosis": admin.VERTICAL,
        "advised_to_fast": admin.VERTICAL,
        "agree_to_p3": admin.VERTICAL,
        "alcoholism": admin.VERTICAL,
        "already_fasted": admin.VERTICAL,
        "art_six_months": admin.VERTICAL,
        "congestive_heart_failure": admin.VERTICAL,
        "continue_part_two": admin.VERTICAL,
        "creatinine_performed": admin.VERTICAL,
        "creatinine_units": admin.VERTICAL,
        "ethnicity": admin.VERTICAL,
        "fasting": admin.VERTICAL,
        "fbg2_units": admin.VERTICAL,
        "fbg_units": admin.VERTICAL,
        "gender": admin.VERTICAL,
        "has_dm": admin.VERTICAL,
        "hba1c_performed": admin.VERTICAL,
        "hiv_pos": admin.VERTICAL,
        "liver_disease": admin.VERTICAL,
        "lives_nearby": admin.VERTICAL,
        "meta_phase_two": admin.VERTICAL,
        "metformin_sensitivity": admin.VERTICAL,
        "ogtt2_units": admin.VERTICAL,
        "ogtt_units": admin.VERTICAL,
        "on_dm_medication": admin.VERTICAL,
        "on_rx_stable": admin.VERTICAL,
        "p3_ltfu": admin.VERTICAL,
        "pregnant": admin.VERTICAL,
        "renal_function_condition": admin.VERTICAL,
        "repeat_fasting": admin.VERTICAL,
        "repeat_glucose_opinion": admin.VERTICAL,
        "repeat_glucose_performed": admin.VERTICAL,
        "screening_consent": admin.VERTICAL,
        "selection_method": admin.VERTICAL,
        "severe_htn": admin.VERTICAL,
        "staying_nearby_12": admin.VERTICAL,
        "staying_nearby_6": admin.VERTICAL,
        "tissue_hypoxia_condition": admin.VERTICAL,
        "unsuitable_agreed": admin.VERTICAL,
        "unsuitable_for_study": admin.VERTICAL,
        "urine_bhcg_performed": admin.VERTICAL,
        "urine_bhcg_value": admin.VERTICAL,
        "vl_undetectable": admin.VERTICAL,
    }

    def post_url_on_delete_kwargs(self, request, obj):
        return {}

    @staticmethod
    def demographics(obj=None):
        data = [
            f"{obj.get_gender_display()} {obj.age_in_years}yrs",
            f"Initials: {obj.initials.upper()}<BR>",
            f"Hospital ID: {obj.hospital_identifier}",
        ]
        if obj.repeat_glucose_opinion == YES:
            data.append(f"Contact #: {obj.contact_number or '--'}")
        return format_html("<BR>".join(data))

    def reasons(self, obj=None):
        if not obj.reasons_ineligible:
            return self.dashboard(obj)
        return format_reasons_ineligible(obj.reasons_ineligible, delimiter="<BR>")

    def eligibility_status(self, obj=None):
        eligibility = MetaEligibility(obj, update_model=False)
        screening_listboard_url = reverse(
            url_names.get(self.subject_listboard_url_name), args=(obj.screening_identifier,)
        )
        context = dict(
            title="Go to screening listboard",
            url=f"{screening_listboard_url}?q={obj.screening_identifier}",
            label="Screening",
        )
        button = render_to_string("dashboard_button.html", context=context)
        return format_html(button + "<BR>" + eligibility.eligibility_status(add_urls=True))

    def dashboard(self, obj=None, label=None):
        try:
            url = reverse(
                self.get_subject_dashboard_url_name(),
                kwargs=self.get_subject_dashboard_url_kwargs(obj),
            )
        except NoReverseMatch:
            url = reverse(url_names.get("screening_listboard_url"), kwargs={})
            context = dict(
                title="Go to screening listboard",
                url=f"{url}?q={obj.screening_identifier}",
                label=label,
            )
        else:
            context = dict(title="Go to subject dashboard", url=url, label=label)
        return render_to_string("dashboard_button.html", context=context)



from django import forms
from edc_action_item.forms import ActionItemCrfFormMixin
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_lab_panel.panels import fbc_panel
from edc_lab_results.form_validator_mixins import BloodResultsFormValidatorMixin

from ...models import BloodResultsFbc


class BloodResultsFbcFormValidator(BloodResultsFormValidatorMixin, CrfFormValidator):
    panel = fbc_panel


class BloodResultsFbcForm(ActionItemCrfFormMixin, CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = BloodResultsFbcFormValidator

    class Meta(ActionItemCrfFormMixin.Meta):
        model = BloodResultsFbc
        fields = "__all__"



# Generated by Django 3.2.11 on 2022-04-12 20:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0104_auto_20220412_2151"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalstudymedication",
            name="refill_date",
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name="studymedication",
            name="refill_date",
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name="historicalstudymedication",
            name="order_next",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=15,
                verbose_name="Order refill for next scheduled visit?",
            ),
        ),
        migrations.AlterField(
            model_name="studymedication",
            name="order_next",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=15,
                verbose_name="Order refill for next scheduled visit?",
            ),
        ),
    ]



from django.contrib import admin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_ae_admin
from ..forms import AeLocalReviewForm
from ..models import AeLocalReview
from .modeladmin_mixins import AeReviewModelAdminMixin


@admin.register(AeLocalReview, site=meta_ae_admin)
class AeLocalReviewAdmin(AeReviewModelAdminMixin, SimpleHistoryAdmin):

    form = AeLocalReviewForm



# Generated by Django 3.2.11 on 2022-03-04 19:33

import edc_model.validators.date
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0096_auto_20220304_0501"),
    ]

    operations = [
        migrations.AlterField(
            model_name="glucose",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="glucose",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
    ]



from django.apps import apps as django_apps
from django.test import TestCase
from edc_sites.models import CurrentSiteManager


class TestManagers(TestCase):
    def test_models(self):
        app_config = django_apps.get_app_config("meta_prn")
        for model_cls in app_config.get_models():
            if "historical" not in model_cls._meta.label_lower:
                self.assertEqual(
                    model_cls._default_manager.__class__,
                    CurrentSiteManager,
                    msg=f"Model is {model_cls}",
                )



# Generated by Django 2.2.7 on 2020-01-18 15:54

import edc_vitals.models.fields.blood_pressure
import edc_vitals.models.fields.weight
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0010_auto_20191213_0314"),
    ]

    operations = [
        migrations.AlterField(
            model_name="followupvitals",
            name="dia_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(null=True),
        ),
        migrations.AlterField(
            model_name="followupvitals",
            name="sys_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(null=True),
        ),
        migrations.AlterField(
            model_name="followupvitals",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(),
        ),
        migrations.AlterField(
            model_name="historicalfollowupvitals",
            name="dia_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(null=True),
        ),
        migrations.AlterField(
            model_name="historicalfollowupvitals",
            name="sys_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(null=True),
        ),
        migrations.AlterField(
            model_name="historicalfollowupvitals",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(),
        ),
        migrations.AlterField(
            model_name="historicalphysicalexam",
            name="dia_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(null=True),
        ),
        migrations.AlterField(
            model_name="historicalphysicalexam",
            name="sys_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(null=True),
        ),
        migrations.AlterField(
            model_name="historicalphysicalexam",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(),
        ),
        migrations.AlterField(
            model_name="physicalexam",
            name="dia_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(null=True),
        ),
        migrations.AlterField(
            model_name="physicalexam",
            name="sys_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(null=True),
        ),
        migrations.AlterField(
            model_name="physicalexam",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(),
        ),
    ]



# Generated by Django 3.2.11 on 2022-03-24 00:04

import django.core.validators
import django_crypto_fields.fields.encrypted_text_field
import edc_model.validators.date
import edc_protocol.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0101_auto_20220316_2147"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="delivery",
            name="informant_contact",
        ),
        migrations.RemoveField(
            model_name="delivery",
            name="informant_is_patient",
        ),
        migrations.RemoveField(
            model_name="historicaldelivery",
            name="informant_contact",
        ),
        migrations.RemoveField(
            model_name="historicaldelivery",
            name="informant_is_patient",
        ),
        migrations.AddField(
            model_name="delivery",
            name="info_available",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                help_text="If NO, please explain below",
                max_length=5,
                verbose_name="Were you able to obtain a report on the delivery?",
            ),
        ),
        migrations.AddField(
            model_name="delivery",
            name="info_not_available_reason",
            field=models.TextField(
                null=True,
                verbose_name="If the report was not available, please explain?",
            ),
        ),
        migrations.AddField(
            model_name="delivery",
            name="info_source",
            field=models.CharField(
                choices=[
                    ("patient", "Study participant"),
                    ("OTHER", "Other"),
                    ("N/A", "Not applicable"),
                ],
                default="patient",
                max_length=25,
                verbose_name="Who / what is the MAIN source of this information?",
            ),
        ),
        migrations.AddField(
            model_name="delivery",
            name="info_source_other",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
                verbose_name="If not reported from study participant, please give the name and contact details of the informant.",
            ),
        ),
        migrations.AddField(
            model_name="historicaldelivery",
            name="info_available",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                help_text="If NO, please explain below",
                max_length=5,
                verbose_name="Were you able to obtain a report on the delivery?",
            ),
        ),
        migrations.AddField(
            model_name="historicaldelivery",
            name="info_not_available_reason",
            field=models.TextField(
                null=True,
                verbose_name="If the report was not available, please explain?",
            ),
        ),
        migrations.AddField(
            model_name="historicaldelivery",
            name="info_source",
            field=models.CharField(
                choices=[
                    ("patient", "Study participant"),
                    ("OTHER", "Other"),
                    ("N/A", "Not applicable"),
                ],
                default="patient",
                max_length=25,
                verbose_name="Who / what is the MAIN source of this information?",
            ),
        ),
        migrations.AddField(
            model_name="historicaldelivery",
            name="info_source_other",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
                verbose_name="If not reported from study participant, please give the name and contact details of the informant.",
            ),
        ),
        migrations.AddField(
            model_name="historicalmedicationadherence",
            name="pill_count_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=5,
                verbose_name="Was a pill count performed",
            ),
        ),
        migrations.AddField(
            model_name="medicationadherence",
            name="pill_count_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=5,
                verbose_name="Was a pill count performed",
            ),
        ),
        migrations.AlterField(
            model_name="delivery",
            name="delivery_datetime",
            field=models.DateTimeField(
                help_text="If TIME unknown, estimate",
                null=True,
                validators=[
                    edc_model.validators.date.datetime_not_future,
                    edc_protocol.validators.datetime_not_before_study_start,
                ],
                verbose_name="Date and time of delivery :",
            ),
        ),
        migrations.AlterField(
            model_name="delivery",
            name="delivery_ga",
            field=models.IntegerField(
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(45),
                ],
                verbose_name="Gestational age at delivery",
            ),
        ),
        migrations.AlterField(
            model_name="delivery",
            name="delivery_location",
            field=models.CharField(
                choices=[
                    ("home", "At home"),
                    ("hospital_clinic", "Hospital / Clinic"),
                    ("OTHER", "Other location, specify"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="Where did the delivery occur?",
            ),
        ),
        migrations.AlterField(
            model_name="delivery",
            name="delivery_time_estimated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=3,
                verbose_name="Is the delivery TIME estimated?",
            ),
        ),
        migrations.AlterField(
            model_name="delivery",
            name="fetal_outcome_count",
            field=models.IntegerField(
                help_text="Each to be reported individually below",
                null=True,
                verbose_name="Number of births / fetal / neonatal outcomes",
            ),
        ),
        migrations.AlterField(
            model_name="delivery",
            name="gm_treated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=5,
                verbose_name="Was the participant treated for gestational diabetes?",
            ),
        ),
        migrations.AlterField(
            model_name="delivery",
            name="informant_relation",
            field=models.CharField(
                choices=[
                    ("husband_wife", "Husband/wife"),
                    ("Parent", "Parent"),
                    ("child", "Child"),
                    ("unknown", "Unknown"),
                    ("OTHER", "Other"),
                    ("hospital_clinic", "Hospital / Clinic records"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="Informants relationship to the participant?",
            ),
        ),
        migrations.AlterField(
            model_name="delivery",
            name="maternal_outcome",
            field=models.CharField(
                choices=[
                    ("no_complications", "No complications"),
                    ("complications_full_recovery", "Complications with full recovery"),
                    (
                        "complications_ongoing_recovery",
                        "Complications with ongoing recovery",
                    ),
                    ("dead", "Maternal mortality"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=50,
                verbose_name="What was the maternal outcome of the pregnancy?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="delivery_datetime",
            field=models.DateTimeField(
                help_text="If TIME unknown, estimate",
                null=True,
                validators=[
                    edc_model.validators.date.datetime_not_future,
                    edc_protocol.validators.datetime_not_before_study_start,
                ],
                verbose_name="Date and time of delivery :",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="delivery_ga",
            field=models.IntegerField(
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(45),
                ],
                verbose_name="Gestational age at delivery",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="delivery_location",
            field=models.CharField(
                choices=[
                    ("home", "At home"),
                    ("hospital_clinic", "Hospital / Clinic"),
                    ("OTHER", "Other location, specify"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="Where did the delivery occur?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="delivery_time_estimated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=3,
                verbose_name="Is the delivery TIME estimated?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="fetal_outcome_count",
            field=models.IntegerField(
                help_text="Each to be reported individually below",
                null=True,
                verbose_name="Number of births / fetal / neonatal outcomes",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="gm_treated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=5,
                verbose_name="Was the participant treated for gestational diabetes?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="informant_relation",
            field=models.CharField(
                choices=[
                    ("husband_wife", "Husband/wife"),
                    ("Parent", "Parent"),
                    ("child", "Child"),
                    ("unknown", "Unknown"),
                    ("OTHER", "Other"),
                    ("hospital_clinic", "Hospital / Clinic records"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="Informants relationship to the participant?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="maternal_outcome",
            field=models.CharField(
                choices=[
                    ("no_complications", "No complications"),
                    ("complications_full_recovery", "Complications with full recovery"),
                    (
                        "complications_ongoing_recovery",
                        "Complications with ongoing recovery",
                    ),
                    ("dead", "Maternal mortality"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=50,
                verbose_name="What was the maternal outcome of the pregnancy?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="last_alive_date",
            field=models.DateField(
                blank=True,
                help_text="Will be validated on the Death report",
                null=True,
                validators=[
                    edc_protocol.validators.date_not_before_study_start,
                    edc_model.validators.date.date_not_future,
                ],
                verbose_name="Date participant last known alive",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("alive", "Alive"),
                    ("dead", "Deceased"),
                    ("unknown", "Unknown"),
                ],
                default="alive",
                help_text="If YES, submit Death report",
                max_length=10,
                null=True,
                verbose_name="Participant's survival status",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisit",
            name="last_alive_date",
            field=models.DateField(
                blank=True,
                help_text="Will be validated on the Death report",
                null=True,
                validators=[
                    edc_protocol.validators.date_not_before_study_start,
                    edc_model.validators.date.date_not_future,
                ],
                verbose_name="Date participant last known alive",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisit",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("alive", "Alive"),
                    ("dead", "Deceased"),
                    ("unknown", "Unknown"),
                ],
                default="alive",
                help_text="If YES, submit Death report",
                max_length=10,
                null=True,
                verbose_name="Participant's survival status",
            ),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-21 22:34

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("meta_screening", "0005_auto_20191021_0353")]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="Time oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="in mmol/L",
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(300),
                ],
                verbose_name="Blood glucose level 2-hours after glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_two_hr_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="Time blood glucose levels 2-hours measured",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="Time oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="in mmol/L",
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(300),
                ],
                verbose_name="Blood glucose level 2-hours after glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_two_hr_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="Time blood glucose levels 2-hours measured",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="Time oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="in mmol/L",
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(300),
                ],
                verbose_name="Blood glucose level 2-hours after glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_two_hr_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="Time blood glucose levels 2-hours measured",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="Time oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="in mmol/L",
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(300),
                ],
                verbose_name="Blood glucose level 2-hours after glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_two_hr_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="Time blood glucose levels 2-hours measured",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="Time oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="in mmol/L",
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(300),
                ],
                verbose_name="Blood glucose level 2-hours after glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_two_hr_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="Time blood glucose levels 2-hours measured",
            ),
        ),
    ]



from django.contrib import admin
from edc_action_item import ActionItemModelAdminMixin
from edc_data_manager.modeladmin_mixins import DataManagerModelAdminMixin
from edc_egfr.admin import EgfrDropNotificationAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import EgfrDropNotificationForm
from ..models import EgfrDropNotification
from .modeladmin import CrfModelAdminMixin


@admin.register(EgfrDropNotification, site=meta_subject_admin)
class EgfrDropNotificationAdmin(
    EgfrDropNotificationAdminMixin,
    DataManagerModelAdminMixin,
    CrfModelAdminMixin,
    ActionItemModelAdminMixin,
    SimpleHistoryAdmin,
):

    form = EgfrDropNotificationForm



from django.contrib import admin
from edc_adverse_event.modeladmin_mixins import AeSusarModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_ae_admin
from ..forms import AeSusarForm
from ..models import AeSusar


@admin.register(AeSusar, site=meta_ae_admin)
class AeSusarAdmin(AeSusarModelAdminMixin, SimpleHistoryAdmin):

    form = AeSusarForm



from django.core.validators import RegexValidator
from django_crypto_fields.fields import EncryptedCharField
from edc_constants.constants import QUESTION_RETIRED
from edc_model.models import BaseUuidModel
from edc_screening.model_mixins import ScreeningModelMixin
from edc_screening.screening_identifier import (
    ScreeningIdentifier as BaseScreeningIdentifier,
)

from ..model_mixins import (
    CalculatedModelMixin,
    EligibilityModelMixin,
    PartOneFieldsModelMixin,
    PartThreeFieldsModelMixin,
    PartTwoFieldsModelMixin,
)


class SubjectScreeningModelError(Exception):
    pass


class ScreeningIdentifier(BaseScreeningIdentifier):

    template = "S{random_string}"


class SubjectScreening(
    PartOneFieldsModelMixin,
    PartTwoFieldsModelMixin,
    PartThreeFieldsModelMixin,
    EligibilityModelMixin,
    CalculatedModelMixin,
    ScreeningModelMixin,
    BaseUuidModel,
):

    identifier_cls = ScreeningIdentifier

    contact_number = EncryptedCharField(
        validators=[RegexValidator(r"^[0-9\-\(\)\ ]+$", message="Enter a valid number")],
        null=True,
        help_text="Provide a contact number if repeating glucose measures (Encryption: RSA)",
    )

    def save(self, *args, **kwargs):
        if self._meta.label_lower == "meta_screening.subjectscreening":
            raise SubjectScreeningModelError("Unable to save. Save via P1-3 proxy models.")
        self.consent_ability = QUESTION_RETIRED
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = "Subject Screening"
        verbose_name_plural = "Subject Screening"



# Generated by Django 2.2.6 on 2019-11-04 22:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("meta_screening", "0008_auto_20191031_0745")]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="continue_part_two",
            field=models.CharField(
                choices=[("Yes", "Yes (default)"), ("No", "No")],
                default="Yes",
                help_text="<B>Important</B>: This response will be be automatically set to YES if:<BR><BR>- the participant meets the eligibility criteria for part one;<BR><BR>- the eligibility criteria for part two is already complete.<BR>",
                max_length=15,
                verbose_name="Continue with <U>part two</U> of the screening process?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="selection_method",
            field=models.CharField(
                choices=[
                    ("random_sampling", "Random sampling"),
                    ("purposively_selected", "Purposively selected"),
                ],
                default="?",
                max_length=25,
                verbose_name="How was the patient selected from the outpatients CTC?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="continue_part_two",
            field=models.CharField(
                choices=[("Yes", "Yes (default)"), ("No", "No")],
                default="Yes",
                help_text="<B>Important</B>: This response will be be automatically set to YES if:<BR><BR>- the participant meets the eligibility criteria for part one;<BR><BR>- the eligibility criteria for part two is already complete.<BR>",
                max_length=15,
                verbose_name="Continue with <U>part two</U> of the screening process?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="selection_method",
            field=models.CharField(
                choices=[
                    ("random_sampling", "Random sampling"),
                    ("purposively_selected", "Purposively selected"),
                ],
                default="?",
                max_length=25,
                verbose_name="How was the patient selected from the outpatients CTC?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="continue_part_two",
            field=models.CharField(
                choices=[("Yes", "Yes (default)"), ("No", "No")],
                default="Yes",
                help_text="<B>Important</B>: This response will be be automatically set to YES if:<BR><BR>- the participant meets the eligibility criteria for part one;<BR><BR>- the eligibility criteria for part two is already complete.<BR>",
                max_length=15,
                verbose_name="Continue with <U>part two</U> of the screening process?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="selection_method",
            field=models.CharField(
                choices=[
                    ("random_sampling", "Random sampling"),
                    ("purposively_selected", "Purposively selected"),
                ],
                default="?",
                max_length=25,
                verbose_name="How was the patient selected from the outpatients CTC?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="continue_part_two",
            field=models.CharField(
                choices=[("Yes", "Yes (default)"), ("No", "No")],
                default="Yes",
                help_text="<B>Important</B>: This response will be be automatically set to YES if:<BR><BR>- the participant meets the eligibility criteria for part one;<BR><BR>- the eligibility criteria for part two is already complete.<BR>",
                max_length=15,
                verbose_name="Continue with <U>part two</U> of the screening process?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="selection_method",
            field=models.CharField(
                choices=[
                    ("random_sampling", "Random sampling"),
                    ("purposively_selected", "Purposively selected"),
                ],
                default="?",
                max_length=25,
                verbose_name="How was the patient selected from the outpatients CTC?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="continue_part_two",
            field=models.CharField(
                choices=[("Yes", "Yes (default)"), ("No", "No")],
                default="Yes",
                help_text="<B>Important</B>: This response will be be automatically set to YES if:<BR><BR>- the participant meets the eligibility criteria for part one;<BR><BR>- the eligibility criteria for part two is already complete.<BR>",
                max_length=15,
                verbose_name="Continue with <U>part two</U> of the screening process?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="selection_method",
            field=models.CharField(
                choices=[
                    ("random_sampling", "Random sampling"),
                    ("purposively_selected", "Purposively selected"),
                ],
                default="?",
                max_length=25,
                verbose_name="How was the patient selected from the outpatients CTC?",
            ),
            preserve_default=False,
        ),
    ]



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_refusal.forms import ScreeningFormMixin
from edc_screening.modelform_mixins import AlreadyConsentedFormMixin

from ..form_validators import SubjectRefusalFormValidator
from ..models import SubjectRefusal


class SubjectRefusalForm(
    AlreadyConsentedFormMixin, ScreeningFormMixin, FormValidatorMixin, forms.ModelForm
):

    form_validator_cls = SubjectRefusalFormValidator

    def clean(self):
        cleaned_data = super().clean()
        return cleaned_data

    class Meta:
        model = SubjectRefusal
        fields = "__all__"



# Generated by Django 3.2.11 on 2022-07-22 15:25

from django.db import migrations
from tqdm import tqdm

from meta_pharmacy.constants import METFORMIN


def update_rx_missing_medication(apps, schema_editor):
    medication_model_cls = apps.get_model("edc_pharmacy.medication")
    rx_model_cls = apps.get_model("edc_pharmacy.rx")
    medication = medication_model_cls.objects.get(name=METFORMIN)
    total = rx_model_cls.objects.filter(medications__isnull=True).count()
    for obj in tqdm(rx_model_cls.objects.filter(medications__isnull=True), total=total):
        if not obj.medications.all().exists():
            obj.medications.add(medication)


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0131_auto_20220722_0411"),
    ]

    operations = [migrations.RunPython(update_rx_missing_medication)]



# Generated by Django 3.2.13 on 2022-09-29 14:42

from django.db import migrations, models
import edc_action_item.managers
import edc_model.validators.date
import edc_protocol.validators
import edc_sites.models
import edc_utils.date


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0048_auto_20220922_2236"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="endofstudy",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="losstofollowup",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="offschedule",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="offschedulepostnatal",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="offschedulepregnancy",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="offstudymedication",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="pregnancynotification",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="protocolincident",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
            ],
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="offstudy_datetime",
            field=models.DateTimeField(
                default=edc_utils.date.get_utcnow,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Off-study date and time",
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="report_datetime",
            field=models.DateTimeField(editable=False, null=True),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="offstudy_datetime",
            field=models.DateTimeField(
                default=edc_utils.date.get_utcnow,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Off-study date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="report_datetime",
            field=models.DateTimeField(editable=False, null=True),
        ),
    ]



# Generated by Django 3.2.13 on 2022-09-06 22:57

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("edc_adverse_event", "0009_auto_20220907_0157"),
        ("meta_ae", "0014_auto_20220826_0406"),
    ]

    operations = [
        migrations.AddField(
            model_name="aeinitial",
            name="ae_action_classification",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_adverse_event.aeactionclassification",
                verbose_name="Classification of action taken",
            ),
        ),
        migrations.AddField(
            model_name="historicalaeinitial",
            name="ae_action_classification",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_adverse_event.aeactionclassification",
                verbose_name="Classification of action taken",
            ),
        ),
    ]



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_screening.modelform_mixins import AlreadyConsentedFormMixin

from ..form_validators import SubjectScreeningFormValidator
from ..models import SubjectScreening


class SubjectScreeningForm(AlreadyConsentedFormMixin, FormValidatorMixin, forms.ModelForm):

    form_validator_cls = SubjectScreeningFormValidator

    def clean(self):
        cleaned_data = super().clean()
        return cleaned_data

    class Meta:
        model = SubjectScreening
        fields = "__all__"



# Generated by Django 3.2.13 on 2022-09-25 02:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0151_auto_20220918_0508"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="subjectrequisition",
            index=models.Index(
                fields=["subject_visit", "site", "panel", "id"],
                name="meta_subjec_subject_2bb98e_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="subjectrequisition",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_c4a56c_idx",
            ),
        ),
    ]



from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase, override_settings
from edc_constants.constants import FEMALE, MALE
from edc_randomization.site_randomizers import site_randomizers
from edc_registration.models import RegisteredSubject

from meta_edc.meta_version import PHASE_THREE, get_meta_version
from meta_rando.randomizers import RandomizerPhaseThree
from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin


class TestRandomizers(MetaTestCaseMixin, TestCase):

    import_randomization_list = False

    def test_import(self):
        RandomizerPhaseThree.import_list(sid_count_for_tests=10)
        obj = RandomizerPhaseThree.model_cls().objects.all().order_by("sid")[0]
        self.assertEqual(
            [1001, "active", "hindu_mandal", "F"],
            [obj.sid, obj.assignment, obj.site_name, obj.gender],
        )

    @override_settings(META_PHASE=PHASE_THREE)
    def test_randomize_phase_three(self):
        self.assertEqual(get_meta_version(), 3)
        site_randomizers._registry = {}
        site_randomizers.loaded = False
        site_randomizers.register(RandomizerPhaseThree)
        self.subject_visit = self.get_subject_visit(gender=FEMALE)
        try:
            RegisteredSubject.objects.get(
                sid=1001,
                subject_identifier=self.subject_visit.subject_identifier,
                gender=FEMALE,
            )
        except ObjectDoesNotExist:
            self.fail("RegisteredSubject unexpectedly does not exist")

        self.subject_visit = self.get_subject_visit(gender=MALE)
        try:
            RegisteredSubject.objects.get(
                sid=1003,
                subject_identifier=self.subject_visit.subject_identifier,
                gender=MALE,
            )
        except ObjectDoesNotExist:
            self.fail("RegisteredSubject unexpectedly does not exist")

        self.subject_visit = self.get_subject_visit(gender=FEMALE)
        try:
            RegisteredSubject.objects.get(
                sid=1002,
                subject_identifier=self.subject_visit.subject_identifier,
                gender=FEMALE,
            )
        except ObjectDoesNotExist:
            self.fail("RegisteredSubject unexpectedly does not exist")



from typing import Tuple

from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_action_item import ActionItemModelAdminMixin, action_fieldset_tuple
from edc_data_manager.modeladmin_mixins import DataManagerModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_prn_admin
from ..forms import OffSchedulePregnancyForm
from ..models import OffSchedulePregnancy


@admin.register(OffSchedulePregnancy, site=meta_prn_admin)
class OffSchedulePregnancyAdmin(
    DataManagerModelAdminMixin,
    ActionItemModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):

    instructions = None

    form = OffSchedulePregnancyForm

    fieldsets = (
        (None, {"fields": ("subject_identifier", "offschedule_datetime")}),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    def get_list_display(self, request) -> Tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = ("subject_identifier", "dashboard", "offschedule_datetime")
        return custom_fields + tuple(f for f in list_display if f not in custom_fields)

    def get_list_filter(self, request) -> Tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        custom_fields = ("offschedule_datetime",)
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)

    def get_readonly_fields(self, request, obj=None) -> Tuple[str, ...]:
        custom_fields = ("subject_identifier", "offschedule_datetime")
        return tuple(set(super().get_readonly_fields(request, obj=obj) + custom_fields))



# Generated by Django 3.2.13 on 2022-07-08 18:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0121_auto_20220704_1841"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalmedicationadherence",
            name="pill_count_not_performed_reason",
            field=models.TextField(
                blank=True, null=True, verbose_name="If not performed, please explain ..."
            ),
        ),
        migrations.AlterField(
            model_name="medicationadherence",
            name="pill_count_not_performed_reason",
            field=models.TextField(
                blank=True, null=True, verbose_name="If not performed, please explain ..."
            ),
        ),
    ]



# Generated by Django 3.2.11 on 2022-03-09 19:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0018_auto_20220309_2106"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="birthoutcomes",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ["subject_identifier", "birth_order"],
                "verbose_name": "Birth Outcomes",
                "verbose_name_plural": "Birth Outcomes",
            },
        ),
        migrations.RenameField(
            model_name="birthoutcomes",
            old_name="maternal_identifier",
            new_name="subject_identifier",
        ),
        migrations.RenameField(
            model_name="historicalbirthoutcomes",
            old_name="maternal_identifier",
            new_name="subject_identifier",
        ),
        migrations.AlterUniqueTogether(
            name="birthoutcomes",
            unique_together={("subject_identifier", "birth_order")},
        ),
    ]



from django.contrib import admin
from edc_adverse_event.modeladmin_mixins import AeFollowupModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_ae_admin
from ..forms import AeFollowupForm
from ..models import AeFollowup


@admin.register(AeFollowup, site=meta_ae_admin)
class AeFollowupAdmin(AeFollowupModelAdminMixin, SimpleHistoryAdmin):

    form = AeFollowupForm



from django import forms
from edc_adherence.form_validator_mixin import MedicationAdherenceFormValidatorMixin
from edc_adherence.model_form_mixin import MedicationAdherenceFormMixin
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_model.widgets import SliderWidget

from ..models import MedicationAdherence


class MedicationAdherenceFormValidator(
    MedicationAdherenceFormValidatorMixin, CrfFormValidator
):
    pass


class MedicationAdherenceForm(
    MedicationAdherenceFormMixin, CrfModelFormMixin, forms.ModelForm
):

    form_validator_cls = MedicationAdherenceFormValidator

    visual_score_slider = forms.CharField(
        label="Visual Score", widget=SliderWidget(attrs={"min": 0, "max": 100})
    )

    class Meta:
        model = MedicationAdherence
        fields = "__all__"



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import ActionItemModelAdminMixin, action_fieldset_tuple
from edc_data_manager.modeladmin_mixins import DataManagerModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_prn_admin
from ..forms import LossToFollowupForm
from ..models import LossToFollowup


@admin.register(LossToFollowup, site=meta_prn_admin)
class LossToFollowupAdmin(
    DataManagerModelAdminMixin,
    ActionItemModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):

    form = LossToFollowupForm

    fieldsets = (
        (None, {"fields": ("subject_identifier", "report_datetime")}),
        (
            "Loss to followup",
            {
                "fields": (
                    "last_seen_datetime",
                    "number_consecutive_missed_visits",
                    "last_missed_visit_datetime",
                    "home_visited",
                    "home_visit_detail",
                    "loss_category",
                    "loss_category_other",
                    "comment",
                )
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_identifier",
        "dashboard",
        "last_seen_datetime",
        "number_consecutive_missed_visits",
        "home_visited",
    )

    list_filter = (
        "last_seen_datetime",
        "last_missed_visit_datetime",
        "number_consecutive_missed_visits",
    )

    radio_fields = {
        "home_visited": admin.VERTICAL,
        "loss_category": admin.VERTICAL,
    }



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset

from ..admin_site import meta_subject_admin
from ..forms import UrineDipstickTestForm
from ..models import UrineDipstickTest
from .modeladmin import CrfModelAdmin


@admin.register(UrineDipstickTest, site=meta_subject_admin)
class UrineDipstickTestAdmin(CrfModelAdmin):

    form = UrineDipstickTestForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_visit",
                    "report_datetime",
                    "performed",
                    "not_performed_reason",
                    "ketones",
                    "protein",
                    "glucose",
                )
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "performed": admin.VERTICAL,
        "ketones": admin.VERTICAL,
        "protein": admin.VERTICAL,
        "glucose": admin.VERTICAL,
    }



# Generated by Django 4.0.5 on 2022-06-26 13:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0028_historicalpregnancynotification_bhcg_date_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="edd",
            field=models.DateField(verbose_name="Estimated date of delivery"),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="edd",
            field=models.DateField(verbose_name="Estimated date of delivery"),
        ),
    ]



# Generated by Django 3.0.6 on 2020-05-16 22:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_lists", "0007_auto_20200516_2356"),
        ("meta_subject", "0034_auto_20200517_0125"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalmissedvisit",
            name="contact_made",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=25,
                verbose_name="Was contact finally made with the participant?",
            ),
        ),
        migrations.AlterField(
            model_name="missedvisit",
            name="contact_made",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=25,
                verbose_name="Was contact finally made with the participant?",
            ),
        ),
        migrations.AlterField(
            model_name="missedvisit",
            name="missed_reasons",
            field=models.ManyToManyField(
                blank=True, to="meta_lists.SubjectVisitMissedReasons"
            ),
        ),
    ]



# Generated by Django 3.2.13 on 2022-08-26 13:40
import sys

from django.db import migrations
from edc_registration.models import RegisteredSubject
from edc_utils import get_utcnow
from tqdm import tqdm

from meta_rando.models import RandomizationList


def update_missing_sid_in_registeredsubject(apps, schema_editor):
    registered_subjects = RegisteredSubject.objects.filter(sid__isnull=True)
    sys.stdout.write(
        f"\n    (*) Found {registered_subjects.count()} records to update ... \n\n"
    )
    for rs_obj in tqdm(registered_subjects, total=registered_subjects.count()):
        rando_obj = RandomizationList.objects.get(subject_identifier=rs_obj.subject_identifier)
        rs_obj.sid = rando_obj.sid
        rs_obj.user_modified = "erikvw"
        rs_obj.modified = get_utcnow()
        rs_obj.comment = (
            f"{rs_obj.comment or ''}[manually updated SID from RandomizationList "
            "(meta_rando.migrations.0003)]."
        )
        rs_obj.save()
    sys.stdout.write(f"\n    Done\n")


class Migration(migrations.Migration):

    dependencies = [
        ("meta_rando", "0002_auto_20220704_1841"),
    ]

    operations = [migrations.RunPython(update_missing_sid_in_registeredsubject)]



# Generated by Django 4.0.5 on 2022-06-27 14:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0046_historicalscreeningpartone_hba1c_datetime_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="appt_datetime_repeat",
            field=models.DateTimeField(
                blank=True,
                help_text="must be at least 3 days after the first glucose measures (FBG, OGTT)",
                null=True,
                verbose_name="Appointment date for repeat glucose testing",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="appt_datetime_repeat",
            field=models.DateTimeField(
                blank=True,
                help_text="must be at least 3 days after the first glucose measures (FBG, OGTT)",
                null=True,
                verbose_name="Appointment date for repeat glucose testing",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="appt_datetime_repeat",
            field=models.DateTimeField(
                blank=True,
                help_text="must be at least 3 days after the first glucose measures (FBG, OGTT)",
                null=True,
                verbose_name="Appointment date for repeat glucose testing",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="appt_datetime_repeat",
            field=models.DateTimeField(
                blank=True,
                help_text="must be at least 3 days after the first glucose measures (FBG, OGTT)",
                null=True,
                verbose_name="Appointment date for repeat glucose testing",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="appt_datetime_repeat",
            field=models.DateTimeField(
                blank=True,
                help_text="must be at least 3 days after the first glucose measures (FBG, OGTT)",
                null=True,
                verbose_name="Appointment date for repeat glucose testing",
            ),
        ),
    ]



# Generated by Django 3.0.4 on 2020-04-17 00:29

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0019_auto_20200417_0315"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="coronakap",
            name="dm_aware",
        ),
        migrations.RemoveField(
            model_name="coronakap",
            name="months_on_art",
        ),
        migrations.RemoveField(
            model_name="historicalcoronakap",
            name="dm_aware",
        ),
        migrations.RemoveField(
            model_name="historicalcoronakap",
            name="months_on_art",
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="employment",
            field=models.CharField(
                choices=[
                    ("professional", "Professional (e.g. office"),
                    ("labourer", "Labourer"),
                    ("housewife_retired", "Housewife, Retired"),
                    ("self_employed", "Small business, self-employed"),
                    ("OTHER", "Other, specify below"),
                ],
                max_length=25,
                verbose_name="Employment",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="employment_status",
            field=models.CharField(
                choices=[
                    ("working_for_pay", "Working for pay"),
                    ("unemployed", "Unemployed"),
                    (
                        "not_working_for_pay",
                        "Not working for pay (housewife, retired...)",
                    ),
                ],
                max_length=25,
                verbose_name="Are you employed",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="family_infection_likelihood",
            field=models.CharField(
                choices=[
                    ("very", "Very likely"),
                    ("somewhat", "Somewhat likely"),
                    ("unlikely", "Not very likely, unlikely"),
                    ("not_at_all", "Not at all"),
                ],
                max_length=25,
                verbose_name="How likely do you think it is that you or someone in your family will get sick from the coronavirus",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="married",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Are you currently married?",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="perceived_threat",
            field=models.IntegerField(
                help_text="On a scale from 1-10",
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(10),
                ],
                verbose_name="On a scale from 1-10, how serious of a public health threat is coronavirus",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="personal_infection_likelihood",
            field=models.CharField(
                choices=[
                    ("very", "Very likely"),
                    ("somewhat", "Somewhat likely"),
                    ("unlikely", "Not very likely, unlikely"),
                    ("not_at_all", "Not at all"),
                ],
                max_length=25,
                verbose_name="How likely do you think it is that you will get corona virus",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="employment",
            field=models.CharField(
                choices=[
                    ("professional", "Professional (e.g. office"),
                    ("labourer", "Labourer"),
                    ("housewife_retired", "Housewife, Retired"),
                    ("self_employed", "Small business, self-employed"),
                    ("OTHER", "Other, specify below"),
                ],
                max_length=25,
                verbose_name="Employment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="employment_status",
            field=models.CharField(
                choices=[
                    ("working_for_pay", "Working for pay"),
                    ("unemployed", "Unemployed"),
                    (
                        "not_working_for_pay",
                        "Not working for pay (housewife, retired...)",
                    ),
                ],
                max_length=25,
                verbose_name="Are you employed",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="family_infection_likelihood",
            field=models.CharField(
                choices=[
                    ("very", "Very likely"),
                    ("somewhat", "Somewhat likely"),
                    ("unlikely", "Not very likely, unlikely"),
                    ("not_at_all", "Not at all"),
                ],
                max_length=25,
                verbose_name="How likely do you think it is that you or someone in your family will get sick from the coronavirus",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="married",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Are you currently married?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="perceived_threat",
            field=models.IntegerField(
                help_text="On a scale from 1-10",
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(10),
                ],
                verbose_name="On a scale from 1-10, how serious of a public health threat is coronavirus",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="personal_infection_likelihood",
            field=models.CharField(
                choices=[
                    ("very", "Very likely"),
                    ("somewhat", "Somewhat likely"),
                    ("unlikely", "Not very likely, unlikely"),
                    ("not_at_all", "Not at all"),
                ],
                max_length=25,
                verbose_name="How likely do you think it is that you will get corona virus",
            ),
        ),
    ]



from .end_of_study_form import EndOfStudyForm
from .loss_to_followup_form import LossToFollowupForm
from .off_study_medication_form import OffStudyMedicationForm
from .offschedule_form import OffScheduleForm
from .offschedule_pregnancy_form import OffSchedulePregnancyForm
from .pregnancy_notification_form import PregnancyNotificationForm
from .protocol_incident_form import ProtocolIncidentForm
from .subject_transfer_form import SubjectTransferForm



from django import forms
from edc_constants.constants import YES
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin

from ..models import PhysicalExam


class PhysicalExamFormValidator(CrfFormValidator):
    def clean(self):

        self.required_if(
            YES,
            field="irregular_heartbeat",
            field_required="irregular_heartbeat_description",
        )

        self.required_if(
            YES,
            field="abdominal_tenderness",
            field_required="abdominal_tenderness_description",
        )


class PhysicalExamForm(CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = PhysicalExamFormValidator

    class Meta:
        model = PhysicalExam
        fields = "__all__"



# Generated by Django 3.2.13 on 2022-09-22 19:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pharmacy", "0015_auto_20220913_2139"),
        ("meta_prn", "0047_auto_20220826_0406"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="offschedule",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Off-schedule",
                "verbose_name_plural": "Off-schedule",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedulepostnatal",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Off-schedule: post-natal",
                "verbose_name_plural": "Off-schedule: post-natal",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedulepregnancy",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Off-schedule: Pregnancy",
                "verbose_name_plural": "Off-schedule: Pregnancy",
            },
        ),
        migrations.AlterModelOptions(
            name="onschedule",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
            },
        ),
        migrations.AlterModelOptions(
            name="onschedulepregnancy",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
            },
        ),
        migrations.AddField(
            model_name="offstudymedication",
            name="medications",
            field=models.ManyToManyField(
                limit_choices_to={"name": "metformin"}, to="edc_pharmacy.Medication"
            ),
        ),
    ]



# Generated by Django 2.2.6 on 2019-11-07 02:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("meta_screening", "0013_auto_20191107_0442")]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_ogtt_two_hr",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose level 2-hours",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="creatinine",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fasting_glucose_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="<u>Time</u> fasting glucose <u>level</u> measured",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_two_hr_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_ogtt_two_hr",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose level 2-hours",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="creatinine",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fasting_glucose_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="<u>Time</u> fasting glucose <u>level</u> measured",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_two_hr_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_ogtt_two_hr",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose level 2-hours",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="creatinine",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fasting_glucose_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="<u>Time</u> fasting glucose <u>level</u> measured",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_two_hr_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_ogtt_two_hr",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose level 2-hours",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="creatinine",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fasting_glucose_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="<u>Time</u> fasting glucose <u>level</u> measured",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_two_hr_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_ogtt_two_hr",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose level 2-hours",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="creatinine",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fasting_glucose_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="<u>Time</u> fasting glucose <u>level</u> measured",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_two_hr_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
    ]



from django.db import models
from edc_constants.choices import YES_NO
from edc_glucose.model_mixins import FastingModelMixin, FbgModelMixin, OgttModelMixin
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin


class Glucose(
    CrfModelMixin,
    FastingModelMixin,
    FbgModelMixin,
    OgttModelMixin,
    edc_models.BaseUuidModel,
):
    # TODO: diagnosis of diabetes is OGTT 11.1mmol / L ONLY. Triggers EoS form
    # TODO: move IFG to bloogresultglu. Use this form for OGTT only 27/01/2021

    """A user model to capture IFG and OGTT"""

    fbg_performed = models.CharField(
        verbose_name="Was the IFG test performed?",
        max_length=15,
        choices=YES_NO,
    )

    fbg_not_performed_reason = models.CharField(
        verbose_name="If NO, provide reason", max_length=150, null=True, blank=True
    )

    ogtt_performed = models.CharField(
        verbose_name="Was the OGTT test performed?",
        max_length=15,
        choices=YES_NO,
    )

    ogtt_not_performed_reason = models.CharField(
        verbose_name="If NO, provide reason", max_length=150, null=True, blank=True
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Glucose (IFG, OGTT)"
        verbose_name_plural = "Glucose (IFG, OGTT)"



# Generated by Django 3.0.4 on 2020-04-03 00:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0003_auto_20200120_2020"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="loss_category",
            field=models.CharField(
                choices=[], max_length=25, verbose_name="Category of loss to follow up"
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="loss_category",
            field=models.CharField(
                choices=[], max_length=25, verbose_name="Category of loss to follow up"
            ),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-20 03:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("meta_screening", "0002_auto_20191020_0612")]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="advised_to_fast",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Has the patient been advised to return fasted for the second stage of the screening?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="already_fasted",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not applicable, subject is not eligible based on the criteria above",
                    ),
                ],
                default="No",
                max_length=15,
                verbose_name="Has the patient come to the clinic today already fasted?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="advised_to_fast",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Has the patient been advised to return fasted for the second stage of the screening?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="already_fasted",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not applicable, subject is not eligible based on the criteria above",
                    ),
                ],
                default="No",
                max_length=15,
                verbose_name="Has the patient come to the clinic today already fasted?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="advised_to_fast",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Has the patient been advised to return fasted for the second stage of the screening?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="already_fasted",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not applicable, subject is not eligible based on the criteria above",
                    ),
                ],
                default="No",
                max_length=15,
                verbose_name="Has the patient come to the clinic today already fasted?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="advised_to_fast",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Has the patient been advised to return fasted for the second stage of the screening?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="already_fasted",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not applicable, subject is not eligible based on the criteria above",
                    ),
                ],
                default="No",
                max_length=15,
                verbose_name="Has the patient come to the clinic today already fasted?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="advised_to_fast",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Has the patient been advised to return fasted for the second stage of the screening?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="already_fasted",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not applicable, subject is not eligible based on the criteria above",
                    ),
                ],
                default="No",
                max_length=15,
                verbose_name="Has the patient come to the clinic today already fasted?",
            ),
        ),
    ]



from django.db import models
from edc_egfr.model_mixins import EgfrModelMixin
from edc_lab.model_mixins import CrfWithRequisitionModelMixin
from edc_lab_panel.panels import rft_panel
from edc_lab_results import BLOOD_RESULTS_RFT_ACTION
from edc_lab_results.model_mixins import (
    BloodResultsModelMixin,
    CreatinineModelMixin,
    UreaModelMixin,
    UricAcidModelMixin,
)
from edc_model.models import BaseUuidModel

from ...model_mixins import CrfWithActionModelMixin


class BloodResultsRft(
    CrfWithActionModelMixin,
    CreatinineModelMixin,
    EgfrModelMixin,
    UreaModelMixin,
    UricAcidModelMixin,
    CrfWithRequisitionModelMixin,
    BloodResultsModelMixin,
    BaseUuidModel,
):
    action_name = BLOOD_RESULTS_RFT_ACTION
    lab_panel = rft_panel
    egfr_formula_name = "ckd-epi"

    old_egfr_value = models.DecimalField(
        decimal_places=4,
        max_digits=8,
        null=True,
        editable=False,
        help_text="incorrect ckd-epi calculation (w/ 1.150 as ethnicity factor)",
    )

    old_egfr_drop_value = models.DecimalField(
        decimal_places=4,
        max_digits=10,
        null=True,
        editable=False,
        help_text="incorrect ckd-epi calculation (w/ 1.150 as ethnicity factor)",
    )

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Blood Result: RFT"
        verbose_name_plural = "Blood Results: RFT"



# Generated by Django 4.0.5 on 2022-06-28 19:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0051_alter_historicalscreeningpartone_advised_to_fast_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="p3_ltfu",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("PENDING", "Decision pending"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Only applicable if the patient missed the appointment for the second stage of screening (P3), several attempts have been made to contact the patient, and the patient has not started P3. See above",
                max_length=15,
                verbose_name="Consider the patient 'lost to screening' for now?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="p3_ltfu",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("PENDING", "Decision pending"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Only applicable if the patient missed the appointment for the second stage of screening (P3), several attempts have been made to contact the patient, and the patient has not started P3. See above",
                max_length=15,
                verbose_name="Consider the patient 'lost to screening' for now?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="p3_ltfu",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("PENDING", "Decision pending"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Only applicable if the patient missed the appointment for the second stage of screening (P3), several attempts have been made to contact the patient, and the patient has not started P3. See above",
                max_length=15,
                verbose_name="Consider the patient 'lost to screening' for now?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="p3_ltfu",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("PENDING", "Decision pending"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Only applicable if the patient missed the appointment for the second stage of screening (P3), several attempts have been made to contact the patient, and the patient has not started P3. See above",
                max_length=15,
                verbose_name="Consider the patient 'lost to screening' for now?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="p3_ltfu",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("PENDING", "Decision pending"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Only applicable if the patient missed the appointment for the second stage of screening (P3), several attempts have been made to contact the patient, and the patient has not started P3. See above",
                max_length=15,
                verbose_name="Consider the patient 'lost to screening' for now?",
            ),
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import FollowupVitalsForm
from ..models import FollowupVitals
from .fields import get_blood_pressure_fields, get_respiratory_o2_fields
from .modeladmin import CrfModelAdminMixin


@admin.register(FollowupVitals, site=meta_subject_admin)
class FollowupVitalsAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):

    form = FollowupVitalsForm

    additional_instructions = [
        "If participant is pregnant, complete the action linked CRF `Pregnancy notification`."
    ]

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Vitals",
            {
                "description": (
                    "To be completed by the research nurse. <BR>"
                    "Refer to SOP for blood pressure measurement procedure."
                ),
                "fields": (
                    "weight",
                    *get_blood_pressure_fields(),
                    "heart_rate",
                    *get_respiratory_o2_fields(),
                    "temperature",
                ),
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    filter_horizontal = ()

    radio_fields = {
        "severe_htn": admin.VERTICAL,
    }
